==> ginkgo-1.6.0.tar.gz: ginkgo-1.6.0/include/ginkgo/core/reorder/nested_dissection.hpp <==
/*******************************<GINKGO LICENSE>******************************
Copyright (c) 2017-2023, the Ginkgo authors
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************<GINKGO LICENSE>*******************************/

#ifndef GKO_PUBLIC_CORE_REORDER_NESTED_DISSECTION_HPP_
#define GKO_PUBLIC_CORE_REORDER_NESTED_DISSECTION_HPP_


#include <ginkgo/config.hpp>


#if GKO_HAVE_METIS


#include <memory>
#include <unordered_map>


#include <ginkgo/core/base/abstract_factory.hpp>
#include <ginkgo/core/base/array.hpp>
#include <ginkgo/core/base/dim.hpp>
#include <ginkgo/core/base/lin_op.hpp>
#include <ginkgo/core/base/polymorphic_object.hpp>
#include <ginkgo/core/base/types.hpp>
#include <ginkgo/core/matrix/csr.hpp>
#include <ginkgo/core/matrix/permutation.hpp>


namespace gko {
namespace experimental {
/**
 * @brief The Reorder namespace.
 *
 * @ingroup reorder
 */
namespace reorder {


/**
 * Computes a Nested Dissection (ND) reordering of an input matrix using the
 * METIS library.
 *
 * @tparam ValueType  the type used to store values of the system matrix
 * @tparam IndexType  the type used to store sparsity pattern indices of the
 *                    system matrix
 */
template <typename ValueType, typename IndexType>
class NestedDissection
    : public EnablePolymorphicObject<NestedDissection<ValueType, IndexType>,
                                     LinOpFactory>,
      public EnablePolymorphicAssignment<
          NestedDissection<ValueType, IndexType>> {
public:
    struct parameters_type;
    friend class EnablePolymorphicObject<NestedDissection<ValueType, IndexType>,
                                         LinOpFactory>;
    friend class enable_parameters_type<parameters_type,
                                        NestedDissection<ValueType, IndexType>>;

    using value_type = ValueType;
    using index_type = IndexType;
    using matrix_type = matrix::Csr<value_type, index_type>;
    using permutation_type = matrix::Permutation<index_type>;

    struct parameters_type
        : public enable_parameters_type<
              parameters_type, NestedDissection<ValueType, IndexType>> {
        /**
         * The options to be passed on to METIS, stored as key-value pairs.
         * Any options that are not set here use their default value.
         */
        std::unordered_map<int, int> options;

        /**
         * @copydoc options
         * @return `*this` for chaining
         */
        parameters_type& with_options(std::unordered_map<int, int> options)
        {
            this->options = std::move(options);
            return *this;
        }
    };

    /**
     * Returns the parameters used to construct the factory.
     */
    const parameters_type& get_parameters() { return parameters_; }

    /**
     * @copydoc LinOpFactory::generate
     * @note This function overrides the default LinOpFactory::generate to
     *       return a Permutation instead of a generic LinOp, which would need
     *       to be cast to Permutation again to access its indices.
     *       It is only necessary because smart pointers aren't covariant.
     */
    std::unique_ptr<permutation_type> generate(
        std::shared_ptr<const LinOp> system_matrix) const;

    /** Creates a new parameter_type to set up the factory. */
    static parameters_type build() { return {}; }

protected:
    explicit NestedDissection(std::shared_ptr<const Executor> exec,
                              const parameters_type& params = {});

    std::unique_ptr<LinOp> generate_impl(
        std::shared_ptr<const LinOp> system_matrix) const override;

private:
    parameters_type parameters_;
};


}  // namespace reorder
}  // namespace experimental
}  // namespace gko


#endif  // GKO_HAVE_METIS


#endif  // GKO_PUBLIC_CORE_REORDER_NESTED_DISSECTION_HPP_

==> ginkgo-1.8.0.tar.gz: ginkgo-1.8.0/include/ginkgo/core/reorder/nested_dissection.hpp <==
// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
//
// SPDX-License-Identifier: BSD-3-Clause

#ifndef GKO_PUBLIC_CORE_REORDER_NESTED_DISSECTION_HPP_
#define GKO_PUBLIC_CORE_REORDER_NESTED_DISSECTION_HPP_


#include <ginkgo/config.hpp>


#if GKO_HAVE_METIS


#include <memory>
#include <unordered_map>


#include <ginkgo/core/base/abstract_factory.hpp>
#include <ginkgo/core/base/array.hpp>
#include <ginkgo/core/base/dim.hpp>
#include <ginkgo/core/base/lin_op.hpp>
#include <ginkgo/core/base/polymorphic_object.hpp>
#include <ginkgo/core/base/types.hpp>
#include <ginkgo/core/matrix/csr.hpp>
#include <ginkgo/core/matrix/permutation.hpp>


namespace gko {
namespace experimental {
/**
 * @brief The Reorder namespace.
 *
 * @ingroup reorder
 */
namespace reorder {


/**
 * Computes a Nested Dissection (ND) reordering of an input matrix using the
 * METIS library.
 *
 * @tparam ValueType  the type used to store values of the system matrix
 * @tparam IndexType  the type used to store sparsity pattern indices of the
 *                    system matrix
 */
template <typename ValueType, typename IndexType>
class NestedDissection
    : public EnablePolymorphicObject<NestedDissection<ValueType, IndexType>,
                                     LinOpFactory>,
      public EnablePolymorphicAssignment<
          NestedDissection<ValueType, IndexType>> {
public:
    struct parameters_type;
    friend class EnablePolymorphicObject<NestedDissection<ValueType, IndexType>,
                                         LinOpFactory>;
    friend class enable_parameters_type<parameters_type,
                                        NestedDissection<ValueType, IndexType>>;

    using value_type = ValueType;
    using index_type = IndexType;
    using matrix_type = matrix::Csr<value_type, index_type>;
    using permutation_type = matrix::Permutation<index_type>;

    struct parameters_type
        : public enable_parameters_type<
              parameters_type, NestedDissection<ValueType, IndexType>> {
        /**
         * The options to be passed on to METIS, stored as key-value pairs.
         * Any options that are not set here use their default value.
         */
        std::unordered_map<int, int> options;

        /**
         * @copydoc options
         * @return `*this` for chaining
         */
        parameters_type& with_options(std::unordered_map<int, int> options)
        {
            this->options = std::move(options);
            return *this;
        }
    };

    /**
     * Returns the parameters used to construct the factory.
     *
     * @return the parameters used to construct the factory.
     */
    const parameters_type& get_parameters() { return parameters_; }

    /**
     * @copydoc LinOpFactory::generate
     * @note This function overrides the default LinOpFactory::generate to
     *       return a Permutation instead of a generic LinOp, which would need
     *       to be cast to Permutation again to access its indices.
     *       It is only necessary because smart pointers aren't covariant.
     */
    std::unique_ptr<permutation_type> generate(
        std::shared_ptr<const LinOp> system_matrix) const;

    /** Creates a new parameter_type to set up the factory. */
    static parameters_type build() { return {}; }

protected:
    explicit NestedDissection(std::shared_ptr<const Executor> exec,
                              const parameters_type& params = {});

    std::unique_ptr<LinOp> generate_impl(
        std::shared_ptr<const LinOp> system_matrix) const override;

private:
    parameters_type parameters_;
};


}  // namespace reorder
}  // namespace experimental
}  // namespace gko


#endif  // GKO_HAVE_METIS


#endif  // GKO_PUBLIC_CORE_REORDER_NESTED_DISSECTION_HPP_
